/****************************************************************************
**
** Copyright (C) 2011 LP-Research
** All rights reserved.
** Contact: LP-Research (klaus@lp-research.com)
**
** This file is part of the Open Motion Analysis Toolkit (OpenMAT).
**
** OpenMAT is free software: you can redistribute it and/or modify it under 
** the terms of the GNU General Public License as published by the Free 
** Software Foundation, either version 3 of the License, or (at your option) 
** any later version.
** 
** OpenMAT is distributed in the hope that it will be useful, but WITHOUT 
** ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
** FITNESS FOR A PARTICULAR PURPOSE. See the GNU \ General Public License 
** for more details.
** 
** You should have received a copy of the GNU General Public License along 
** with the OpenMAT library. If not, see <http://www.gnu.org/licenses/>.
**
****************************************************************************/

#ifndef ANDROID_BLUETOOTH
#define ANDROID_BLUETOOTH

#include <iostream>
#include <string>
#include <queue>
#include <vector>

#include <boost/thread/thread.hpp> 

#include "ImuData.h"
#include "MicroMeasure.h"
#include "LpmsIoInterface.h"

using namespace std;

#include <jni.h>
#include <android/log.h>

#include <jni.h>

#define LOGV(...) __android_log_print(ANDROID_LOG_VERBOSE, "LpmsMonitor", __VA_ARGS__)
#define LOGD(...) __android_log_print(ANDROID_LOG_DEBUG, "LpmsMonitor", __VA_ARGS__)
#define LOGI(...) __android_log_print(ANDROID_LOG_INFO, "LpmsMonitor", __VA_ARGS__)
#define LOGW(...) __android_log_print(ANDROID_LOG_WARN, "LpmsMonitor", __VA_ARGS__)
#define LOGE(...) __android_log_print(ANDROID_LOG_ERROR, "LpmsMonitor", __VA_ARGS__) 	

/*! MODBUS receive states */
#define PACKET_ADDRESS0 0
#define PACKET_ADDRESS1 1
#define PACKET_FUNCTION0 2
#define PACKET_FUNCTION1 3
#define PACKET_RAW_DATA 4
#define PACKET_LRC_CHECK0 5
#define PACKET_LRC_CHECK1 6
#define PACKET_END 7
#define PACKET_LENGTH0 8
#define PACKET_LENGTH1 9

class AndroidBluetooth : public LpmsIoInterface {
public:
	/* Constructor. */
	AndroidBluetooth(CalibrationData *configData, JavaVM *thisVm, jobject bluetoothAdapter);

	/* Destructor. */
	~AndroidBluetooth(void);

	/* Connects to LPMS. */
	bool connect(string deviceId);

	/* Closes the connection to LPMS. */
	void close(void);

	/* Polls data from sensor. This method is normally called from SensorManager to retrieve pending
		Bluetooth data. */
	bool pollData(void);

	/* Sends a MODBUS command over the Bluetooth connection. */
	bool sendModbusData(unsigned address, unsigned function, unsigned length, unsigned char *data);

	/* Parses a single MODBUS byte. */
	bool parseModbusByte(unsigned char b);

	/* Returns true if the device has been successfully started. */
	bool deviceStarted(void);

	/* Returns the number of microseconds that should be waited for the device to connect successfully. */
	long long getConnectWait(void);

	/* Lists all connected LPMS Bluetooth devices. The device discovery might take several seconds. */
	static void listDevices(vector<DeviceListItem>* deviceList);

private:
	bool read(char *rxBuffer, unsigned long *bytesReceived); 
	bool write(char *txBuffer, unsigned bufferLength);
	void readThread(void);

	bool isOpen;
	MicroMeasure mm;
	string idNumber;
	string bluetoothAddress;
	jobject bluetoothAdapter;
	JNIEnv* jniEnv;
	JavaVM *thisVm;
	jmethodID connectMethod;
    jmethodID readMethod;
	jmethodID getReadBufferMethod;
	jmethodID getNumberBytesReadMethod;
    jmethodID writeMethod;
	jobject blueObject;
	jclass blueClass;
	jbyteArray jbaOutputBuffer;
	jbyte* jbOutputData;	
	jbyteArray jbaInputBuffer;
	jbyte* jbInputData;		
};

#endif