package com.openmat.lpmsmonitor;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.UUID;
import java.lang.Float.*;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothServerSocket;
import android.bluetooth.BluetoothSocket;
import android.content.Context;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;

// Bluetooth access through Java interface for LPMS. This is accessed
// through the JNI library. Do not modify!
public class BluetoothInterface {
    private static final String TAG = "LpmsMonitor";

    private static final UUID MY_UUID_INSECURE = UUID.fromString("00001101-0000-1000-8000-00805F9B34FB");
    
	public InputStream blueInStream;
	public OutputStream blueOutStream;
	public BluetoothSocket blueSocket;
	
    public BluetoothInterface() {
	}

    public void connect(String address, BluetoothAdapter blueAdapter) {
		Log.e(TAG, "[AndroidBluetooth] Connect to: " + address);

        if (blueAdapter == null) {
			Log.e(TAG, "[AndroidBluetooth] Didn't find Bluetooth adapter");

			return;
        }
		
		blueAdapter.cancelDiscovery();
	
		Log.e(TAG, "[AndroidBluetooth] Getting device");
				
		BluetoothDevice blueDevice;
		try {
			blueDevice = blueAdapter.getRemoteDevice(address);
		} catch (IllegalArgumentException e) {
			Log.e(TAG, "[AndroidBluetooth] Invalid Bluetooth address", e);
			return;
		}

		blueSocket = null;

		Log.e(TAG, "[AndroidBluetooth] Creating socket");
		try {
			blueSocket = blueDevice.createInsecureRfcommSocketToServiceRecord(MY_UUID_INSECURE);
		} catch (IOException e) {
			Log.e(TAG, "[AndroidBluetooth] Socket create() failed", e);
			return;
		}
		
		Log.e(TAG, "[AndroidBluetooth] Trying to connect..");	
		try {
			blueSocket.connect();
		} catch (IOException e) {
			Log.e(TAG, "[AndroidBluetooth] Couldn't connect to device", e);
			return;
		}
		
		Log.e(TAG, "[AndroidBluetooth] Connected!");		
		
		try {
			blueInStream = blueSocket.getInputStream();
			blueOutStream = blueSocket.getOutputStream();
		} catch (IOException e) {
			Log.e(TAG, "[AndroidBluetooth] Streams not created", e);
			return;
		}
    }

	public void writeBuffer(byte[] outputBuffer, int length) {		
		try {
			blueOutStream.write(outputBuffer, 0, length);
		} catch (IOException e) {
			Log.e(TAG, "[AndroidBluetooth] Exception during writeBuffer", e);
		}		
		
	}
	
	public int readBuffer(byte[] buffer) {
		int bytesRead = 0;
		
		try {		
			bytesRead = blueInStream.read(buffer, 0, 5);			
		} catch (IOException e) {
			Log.e(TAG, "Exception during readBuffer", e);
		}
			
		return bytesRead;
	}
	
	public void close() {
		try {
			blueSocket.close();
		} catch (IOException e) {
			Log.e(TAG, "Exception during close()", e);
		}	
	}
}