package com.openmat.lpmsmonitor;

import com.openmat.lpmsmonitor.BluetoothInterface;

import java.util.Timer;
import java.util.TimerTask;
import java.text.DecimalFormat;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.nio.ByteBuffer;
import java.util.LinkedList;
import java.util.Queue;

import android.view.ViewGroup.LayoutParams;
import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.KeyEvent;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.view.View.OnClickListener;
import android.view.inputmethod.EditorInfo;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import android.widget.ToggleButton;
import android.content.Context;
import android.opengl.GLSurfaceView;
import android.view.MotionEvent;

// Main object
public class LpmsMonitor extends Activity {
    private static final boolean D = true;

    private Button startMeasurementButton;
    private Button stopMeasurementButton;

	TextView statusText;
	TextView accelerometerOutput;
	TextView gyroscopeOutput;
	TextView magnetometerOutput;
	EditText addressEdit;
	String currentAddress;
	Timer guiTimer;	
	
	private LpmsSurfaceView glView;
	
	boolean isConnected = false;
	
	// Initilize application and connect to sensor
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if(D) Log.e(LpmsMonitor, "Initializing LpmsMonitor");
				
		if (isConnected == false) {
			// Creates (default) Bluetooth adapter for sensor to connect to
			BluetoothAdapter blueAdapter = BluetoothAdapter.getDefaultAdapter();		
			
			// Initializes the LPMS sensor manager through Java native interface
			initSensorManagerJNI(blueAdapter);
			
			// Adds sensor with Bluetooth ID 00:06:66:45:07:09 to sensor manager
			addSensorJNI("00:06:66:45:07:09");
			
			isConnected = true;
		}

		// Initializes timer to poll data
		guiTimer = new Timer();
		guiTimer.schedule(new TimerTask() {
			@Override
			public void run() {
				timerMethod();
			}
		}, 0, 10);

		// Create the 3D cube
        glView = new LpmsSurfaceView(this);
		
		// Create fullscreen window for application
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        getWindow().setFlags(	WindowManager.LayoutParams.FLAG_FULLSCREEN, 
                                WindowManager.LayoutParams.FLAG_FULLSCREEN);		
		
        setContentView(glView);
    }
	
	// Timer callback
	private void timerMethod() 
	{
		this.runOnUiThread(doSomething);
	}
	
	// String formatter
	public String to3DString(double inValue)
	{
		DecimalFormat threeDec = new DecimalFormat("00.000");
		threeDec.setGroupingUsed(false);
		return threeDec.format(inValue);
	}	

	// Converts 4-Element quaternion to rotation matrix
	public float[] quaternionToMatrix(float[] q)
	{
		float qw = q[0];
		float qx = q[1];
		float qy = q[2];
		float qz = q[3];	
		
		float[] M = new float[16];
		
		M[0] = qw*qw + qx*qx - qy*qy - qz*qz;
		M[1] = 2*qx*qy - 2*qw*qz;
		M[2] = 2*qx*qz + 2*qw*qy;
		M[3] = 0;

		M[4] = 2*qx*qy + 2*qw*qz;
		M[5] = qw*qw - qx*qx + qy*qy - qz*qz;
		M[6] = 2*qy*qz - 2*qw*qx;
		M[7] = 0;

		M[8] = 2*qx*qz - 2*qw*qy;
		M[9] = 2*qy*qz + 2*qw*qx;
		M[10] = qw*qw - qx*qx - qy*qy + qz*qz;
		M[11] = 0;
		
		M[12] = 0;
		M[13] = 0;
		M[14] = 0;
		M[15] = 1.0f;
		
		return M;
	}
	
	// Runs sensor data polling in UI thread (called from timer callback)
	private Runnable doSomething = new Runnable() {
		public void run() {
			getSensorData(glView.lmRenderer.q);
			glView.requestRender();
		}
	};	
	
	// TODO: Following funtions need to be implemented for proper handling of OS task switching etc.
	public void connectHost(View view) 
	{	
	}
	
	public void disconnectHost(View view) 
	{
	}
	
	public void startSensors(View view) 
	{
	}
	
	public void stopSensors(View view) 
	{
	}	

    @Override
    public void onStart() {
        super.onStart();
        if(D) Log.e(TAG, "++ ON START ++");
    }

    @Override
    public synchronized void onResume() {
        super.onResume();
        if(D) Log.e(TAG, "+ ON RESUME +");
    }

    private void setupApplication() {
        Log.d(TAG, "+ SETUP APPLICATION +");
    }

    @Override
    public synchronized void onPause() {
        super.onPause();
        if(D) Log.e(TAG, "- ON PAUSE -");
    }

    @Override
    public void onStop() {
        super.onStop();
        if(D) Log.e(TAG, "-- ON STOP --");
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        if(D) Log.e(TAG, "--- ON DESTROY ---");
    }
	
	// JNI function declaration
	public native void initSensorManagerJNI(BluetoothAdapter blueAdapter);
	public native int addSensorJNI(String address);
	static native void getSensorData(float[] q);
	public native String stringFromJNI();
	
	// Loads LpSensor library to access LPMS sensor functions
	static {
        System.loadLibrary("LpSensorJNI");
    }
}

// Surface class for displaying the 3D cube
class LpmsSurfaceView extends GLSurfaceView {
    public LpmsMonitorRenderer lmRenderer;

	public LpmsSurfaceView(Context context) {
		super(context);
		lmRenderer = new LpmsMonitorRenderer(context);
		
		setRenderer(lmRenderer);	
	}
	
	// By touching the screen, the sensor orientation offset is reset
    @Override 
    public boolean onTouchEvent(MotionEvent e) {
		switch (e.getAction()) {
			case MotionEvent.ACTION_MOVE:
				resetSensorOrientation();
		}
		
		return true;
	}
	
	static native void resetSensorOrientation();
	
	static {
        System.loadLibrary("LpSensorJNI");
    }	
}