package com.openmat.lpmsmonitor;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.io.IOException;
import java.io.InputStream;
import android.opengl.GLSurfaceView;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.opengl.GLUtils;
import android.opengl.Matrix;
import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;
import android.content.Context;
import android.opengl.GLSurfaceView;
import android.opengl.GLU;
import android.util.Log;

// Renders the 3D cube 
public class LpmsMonitorRenderer implements GLSurfaceView.Renderer {
	private FloatBuffer vertexBuffer;
	private int numFaces = 6;
	private FloatBuffer texBuffer; 
	public float[] verticesBuffer;
	
	float[] texCoords = {
		0.0f, 1.0f,
		1.0f, 1.0f,
		0.0f, 0.0f,
		1.0f, 0.0f
	};
   
	private int[] textureIDs = new int[numFaces];	
	
	private float[][] colors = {
		{ 1.0f, 0.5f, 0.0f, 1.0f },
		{ 1.0f, 0.0f, 1.0f, 1.0f },
		{ 0.0f, 1.0f, 0.0f, 1.0f },
		{ 0.0f, 0.0f, 1.0f, 1.0f },
		{ 1.0f, 0.0f, 0.0f, 1.0f }, 
		{ 1.0f, 1.0f, 0.0f, 1.0f }
	};	
	
	public float[] vertices = {
		-1.0f, -1.0f,  1.0f,
		1.0f, -1.0f,  1.0f,
		-1.0f,  1.0f,  1.0f,
		1.0f,  1.0f,  1.0f,

		1.0f, -1.0f, -1.0f, 
		-1.0f, -1.0f, -1.0f,
		1.0f,  1.0f, -1.0f, 
		-1.0f,  1.0f, -1.0f, 

		-1.0f, -1.0f, -1.0f, 
		-1.0f, -1.0f,  1.0f, 
		-1.0f,  1.0f, -1.0f,
		-1.0f,  1.0f,  1.0f,

		1.0f, -1.0f,  1.0f,
		1.0f, -1.0f, -1.0f,
		1.0f,  1.0f,  1.0f,
		1.0f,  1.0f, -1.0f, 

		-1.0f,  1.0f,  1.0f, 
		1.0f,  1.0f,  1.0f,
		-1.0f,  1.0f, -1.0f,
		1.0f,  1.0f, -1.0f,

		-1.0f, -1.0f, -1.0f, 
		1.0f, -1.0f, -1.0f,
		-1.0f, -1.0f,  1.0f, 
		1.0f, -1.0f,  1.0f
	};	
	
	public float[] q;	
	public float[] vertices3;
	
	private static Context context;	
	
	// Constructor
	LpmsMonitorRenderer(Context appContext) {
		context = appContext;
		q = new float[4];
		vertices3 = new float[128];		
	}
  
	// Callback when surface is created. Initialized OpenGL ES environment
    public void onSurfaceCreated(GL10 gl, EGLConfig config) {
		gl.glClearColor(0.6f, 0.6f, 1.0f, 1.0f);
		gl.glClearDepthf(1.0f);
		gl.glEnable(GL10.GL_DEPTH_TEST);
		gl.glDepthFunc(GL10.GL_LEQUAL);
		gl.glHint(GL10.GL_PERSPECTIVE_CORRECTION_HINT, GL10.GL_NICEST); 
		gl.glShadeModel(GL10.GL_SMOOTH);
		gl.glDisable(GL10.GL_DITHER); 
		gl.glEnable(GL10.GL_TEXTURE_2D);
		
		loadTexture(gl);	
		initShapes();		
    }
		
	// Draws one frame with the 3D cube
    public void onDrawFrame(GL10 gl) {
		// Retrieves vertex data by calling JNI funtion (this is where the actual 
		// sensor information is fetched).
		calcVertices(vertices, q, vertices3);
		
		// Inserts new vertices into buffer and draws the scene
		ByteBuffer vbb = ByteBuffer.allocateDirect(vertices3.length * 4);
		vbb.order(ByteOrder.nativeOrder());
		vertexBuffer = vbb.asFloatBuffer();
		vertexBuffer.put(vertices3);
		vertexBuffer.position(0);
				
		gl.glClear(GL10.GL_COLOR_BUFFER_BIT | GL10.GL_DEPTH_BUFFER_BIT);

		gl.glLoadIdentity();
		gl.glTranslatef(0.0f, 0.0f, -6.0f);
		
		gl.glFrontFace(GL10.GL_CCW);
		gl.glEnable(GL10.GL_CULL_FACE);
		gl.glCullFace(GL10.GL_BACK);

		gl.glEnableClientState(GL10.GL_VERTEX_ARRAY);
		gl.glVertexPointer(3, GL10.GL_FLOAT, 0, vertexBuffer);
	
		gl.glEnableClientState(GL10.GL_TEXTURE_COORD_ARRAY);
		gl.glTexCoordPointer(2, GL10.GL_FLOAT, 0, texBuffer);			
		
		for (int i = 0; i < 6; i++) {		
				gl.glDrawArrays(GL10.GL_TRIANGLE_STRIP, i*4, 4);
		}
		
		gl.glDisableClientState(GL10.GL_TEXTURE_COORD_ARRAY);
		gl.glDisableClientState(GL10.GL_VERTEX_ARRAY);
		gl.glDisable(GL10.GL_CULL_FACE);
    }
    
	// OpenGL ES perspective adjustment
    public void onSurfaceChanged(GL10 gl, int width, int height) {
		if (height == 0) height = 1; 
		float aspect = (float) width / height;

		gl.glViewport(0, 0, width, height);

		gl.glMatrixMode(GL10.GL_PROJECTION);
		gl.glLoadIdentity();
		
		GLU.gluPerspective(gl, 45, aspect, 0.1f, 100.f);

		gl.glMatrixMode(GL10.GL_MODELVIEW);
		gl.glLoadIdentity();
    }    	
	
	// Initializes cube shape
	private void initShapes() {
		ByteBuffer vbb = ByteBuffer.allocateDirect(vertices.length * 4);
		vbb.order(ByteOrder.nativeOrder());
		vertexBuffer = vbb.asFloatBuffer();
		vertexBuffer.put(vertices);
		vertexBuffer.position(0);
		
		ByteBuffer tbb = ByteBuffer.allocateDirect(texCoords.length * 4 * numFaces);
		tbb.order(ByteOrder.nativeOrder());
		texBuffer = tbb.asFloatBuffer();
		
		for (int i=0; i<numFaces; i++) {
			texBuffer.put(texCoords);
		}
		
		texBuffer.position(0);		
	}
	
	// Loads cube texture
	public void loadTexture(GL10 gl) {
		gl.glGenTextures(1, textureIDs, 0);

		gl.glBindTexture(GL10.GL_TEXTURE_2D, textureIDs[0]);

		gl.glTexParameterf(GL10.GL_TEXTURE_2D, GL10.GL_TEXTURE_MIN_FILTER, GL10.GL_NEAREST);
		gl.glTexParameterf(GL10.GL_TEXTURE_2D, GL10.GL_TEXTURE_MAG_FILTER, GL10.GL_LINEAR);

		InputStream istream = context.getResources().openRawResource(R.drawable.lpmslogo);
		Bitmap bitmap;
		
		Log.e("LpmsMonitor", "Loading cube texture."); 
		
		try {
			bitmap = BitmapFactory.decodeStream(istream);
		} finally {
			try {
				istream.close();
			} catch(IOException e) {
				Log.e("LpmsMonitor", "Texture streaming has failed."); 			
			}
		}

		GLUtils.texImage2D(GL10.GL_TEXTURE_2D, 0, bitmap, 0);
		bitmap.recycle();
		
		Log.e("LpmsMonitor", "Texture has been loaded."); 
	}	
	
	// Declaration of the native function for calculating the vertex coordinates
	public native void calcVertices(float[] v, float[] q, float[] outV);
	
	static {
        System.loadLibrary("LpSensorJNI");
    }	
}